ENTRY_POINT_V07 = "0x0000000071727De22E5E9d8BAf0edAc6f37da032"
PIMLICO_ESTIMATION_ADDRESS = "0x74cb5e4ee81b86e70f9045036a1c5477de69ee87"

# Correct function selectors
SIMULATE_ENTRY_POINT_SELECTOR = "0xc18f5226"
SIMULATE_HANDLE_OP_LAST_SELECTOR = "0x263934db"  # simulateHandleOpLast selector

